@@ -17,7 +17,8 @@ from shapash.utils.shap_backend import shap_contributions, check_explainer, get_
 from shapash.utils.check import check_model, check_label_dict, check_ypred, check_contribution_object,\
     check_postprocessing, check_features_name
 from shapash.manipulation.select_lines import keep_right_contributions
-from shapash.report.generation import execute_report, export_and_save_report
+from shapash.report import check_report_requirements
+from shapash.manipulation.summarize import create_grouped_features_values
 from .smart_state import SmartState
 from .multi_decorator import MultiDecorator
 from .smart_plotter import SmartPlotter
@@ -124,9 +125,11 @@ class SmartExplainer:
             self.title_story = title_story
         else:
             self.title_story = ''
+        self.features_groups = None
 
     def compile(self, x, model, explainer=None, contributions=None, y_pred=None,
-                preprocessing=None, postprocessing=None, title_story: str = None):
+                preprocessing=None, postprocessing=None, title_story: str = None,
+                features_groups=None):
         """
         The compile method is the first step to understand model and prediction. It performs the sorting
         of contributions, the reverse preprocessing steps and performs all the calculations necessary for
@@ -184,6 +187,16 @@ class SmartExplainer:
         title_story: str (default: None)
             The default title is empty. You can specify a custom title
             which can be used the webapp, or other methods
+        features_groups : dict, optional (default: None)
+            Dictionnary containing features that should be grouped together. This option allows
+            to compute and display the contributions and importance of this group of features.
+            Features that are grouped together will still be displayed in the webapp when clicking
+            on a group.
+
+            >>> {
+            ‘feature_group_1’ : ['feature3', 'feature7', 'feature24'],
+            ‘feature_group_2’ : ['feature1', 'feature12'],
+            }
 
         Example
         --------
@@ -229,6 +242,32 @@ class SmartExplainer:
         self.features_desc = self.check_features_desc()
         if title_story is not None:
             self.title_story = title_story
+        self.features_groups = features_groups
+        if features_groups:
+            self._compile_features_groups(features_groups)
+
+    def _compile_features_groups(self, features_groups):
+        """
+        Performs required computations for groups of features.
+        """
+        # Compute contributions for groups of features
+        self.contributions_groups = self.state.compute_grouped_contributions(self.contributions, features_groups)
+        self.features_imp_groups = None
+        # Update features dict with groups names
+        self._update_features_dict_with_groups(features_groups=features_groups)
+        # Compute t-sne projections for groups of features
+        self.x_pred_groups = create_grouped_features_values(x_pred=self.x_pred, x_init=self.x_init,
+                                                            preprocessing=self.preprocessing,
+                                                            features_groups=self.features_groups,
+                                                            features_dict=self.features_dict,
+                                                            how='dict_of_values')
+        # Compute data attribute for groups of features
+        self.data_groups = self.state.assign_contributions(
+            self.state.rank_contributions(
+                self.contributions_groups,
+                self.x_pred_groups
+            )
+        )
 
     def add(self, y_pred=None, label_dict=None, features_dict=None, title_story: str = None):
         """
@@ -517,6 +556,16 @@ class SmartExplainer:
         for feature in (set(list(self.columns_dict.values())) - set(list(self.features_dict))):
             self.features_dict[feature] = feature
 
+    def _update_features_dict_with_groups(self, features_groups):
+        """
+        Add groups into features dict and inv_features_dict if not present.
+        """
+        for group_name in features_groups.keys():
+            self.features_desc[group_name] = 1000
+            if group_name not in self.features_dict.keys():
+                self.features_dict[group_name] = group_name
+                self.inv_features_dict[group_name] = group_name
+
     def check_contributions(self):
         """
         Check if contributions and prediction set match in terms of shape and index.
@@ -634,7 +683,8 @@ class SmartExplainer:
             features_to_hide=None,
             threshold=None,
             positive=None,
-            max_contrib=None
+            max_contrib=None,
+            display_groups=None
     ):
         """
         The filter method is an important method which allows to summarize the local explainability
@@ -655,26 +705,35 @@ class SmartExplainer:
             If None, hide nothing.
         max_contrib : int, optional (default: None)
             Maximum number of contributions to show.
+        display_groups : bool (default: None)
+            Whether or not to display groups of features. This option is
+            only useful if groups of features are declared when compiling
+            SmartExplainer object.
         """
-        mask = [self.state.init_mask(self.data['contrib_sorted'], True)]
+        display_groups = True if (display_groups is not False and self.features_groups is not None) else False
+        if display_groups:
+            data = self.data_groups
+        else:
+            data = self.data
+        mask = [self.state.init_mask(data['contrib_sorted'], True)]
         if features_to_hide:
             mask.append(
                 self.state.hide_contributions(
-                    self.data['var_dict'],
+                    data['var_dict'],
                     features_list=self.check_features_name(features_to_hide)
                 )
             )
         if threshold:
             mask.append(
                 self.state.cap_contributions(
-                    self.data['contrib_sorted'],
+                    data['contrib_sorted'],
                     threshold=threshold
                 )
             )
         if positive is not None:
             mask.append(
                 self.state.sign_contributions(
-                    self.data['contrib_sorted'],
+                    data['contrib_sorted'],
                     positive=positive
                 )
             )
@@ -682,7 +741,7 @@ class SmartExplainer:
         if max_contrib:
             self.mask = self.state.cutoff_contributions(self.mask, max_contrib=max_contrib)
         self.masked_contributions = self.state.compute_masked_contributions(
-            self.data['contrib_sorted'],
+            data['contrib_sorted'],
             self.mask
         )
         self.mask_params = {
@@ -760,7 +819,8 @@ class SmartExplainer:
             threshold=None,
             positive=None,
             max_contrib=None,
-            proba=False
+            proba=False,
+            use_groups=None
     ):
         """
         The to_pandas method allows to export the summary of local explainability.
@@ -790,6 +850,9 @@ class SmartExplainer:
             Number of contributions to show in the pandas df
         proba : bool, optional (default: False)
             adding proba in output df
+        use_groups : bool (optional)
+            Whether or not to use groups of features contributions (only available if features_groups
+            parameter was not empty when calling compile method).
 
         Returns
         -------
@@ -806,6 +869,11 @@ class SmartExplainer:
         1	3	    0.628911	Sex	        2.0	        0.585475	    Pclass	    1.0	        0.370504
         2	0	    0.543308	Sex	        2.0	        -0.486667	    Pclass	    3.0	        0.255072
         """
+        use_groups = True if (use_groups is not False and self.features_groups is not None) else False
+        if use_groups:
+            data = self.data_groups
+        else:
+            data = self.data
 
         # Classification: y_pred is needed
         if self.y_pred is None:
@@ -815,21 +883,34 @@ class SmartExplainer:
 
         # Apply filter method if necessary
         if all(var is None for var in [features_to_hide, threshold, positive, max_contrib]) \
-                and hasattr(self, 'mask_params'):
+                and hasattr(self, 'mask_params') \
+                and (
+                # if the already computed mask does not have the right shape (this can happen when
+                # we use groups of features once and then use method without groups)
+                (isinstance(data['contrib_sorted'], pd.DataFrame)
+                    and len(data["contrib_sorted"].columns) == len(self.mask.columns))
+                or
+                (isinstance(data['contrib_sorted'], list)
+                    and len(data["contrib_sorted"][0].columns) == len(self.mask[0].columns))
+                ):
             print('to_pandas params: ' + str(self.mask_params))
         else:
             self.filter(features_to_hide=features_to_hide,
                         threshold=threshold,
                         positive=positive,
-                        max_contrib=max_contrib)
-
+                        max_contrib=max_contrib,
+                        display_groups=use_groups)
+        if use_groups:
+            columns_dict = {i: col for i, col in enumerate(self.x_pred_groups.columns)}
+        else:
+            columns_dict = self.columns_dict
         # Summarize information
-        self.data['summary'] = self.state.summarize(
-            self.data['contrib_sorted'],
-            self.data['var_dict'],
-            self.data['x_sorted'],
+        data['summary'] = self.state.summarize(
+            data['contrib_sorted'],
+            data['var_dict'],
+            data['x_sorted'],
             self.mask,
-            self.columns_dict,
+            columns_dict,
             self.features_dict
         )
         # Matching with y_pred
@@ -839,7 +920,7 @@ class SmartExplainer:
         else:
             proba_values = None
 
-        y_pred, summary = keep_right_contributions(self.y_pred, self.data['summary'],
+        y_pred, summary = keep_right_contributions(self.y_pred, data['summary'],
                                                    self._case, self._classes,
                                                    self.label_dict, proba_values)
 
@@ -864,6 +945,8 @@ class SmartExplainer:
             Each Serie: feature importance, One row by feature,
             index of the serie = contributions.columns
         """
+        if self.features_groups is not None and self.features_imp_groups is None:
+            self.features_imp_groups = self.state.compute_features_import(self.contributions_groups)
         if self.features_imp is None or force:
             self.features_imp = self.state.compute_features_import(self.contributions)
 
@@ -965,7 +1048,7 @@ class SmartExplainer:
         self.features_types = {features: str(self.x_pred[features].dtypes) for features in self.x_pred.columns}
 
         listattributes = ["features_dict", "model", "columns_dict", "explainer", "features_types",
-                          "label_dict", "preprocessing", "postprocessing"]
+                          "label_dict", "preprocessing", "postprocessing", "features_groups"]
 
         params_smartpredictor = [self.check_attributes(attribute) for attribute in listattributes]
 
@@ -1027,7 +1110,8 @@ class SmartExplainer:
                         title_description=None,
                         metrics=None,
                         working_dir=None,
-                        notebook_path=None):
+                        notebook_path=None,
+                        kernel_name=None):
         """
         This method will generate an HTML report containing different information about the project.
 
@@ -1052,7 +1136,7 @@ class SmartExplainer:
             Report title.
         title_description : str, optional
             Report title description (as written just below the title).
-        metrics : dict, optional
+        metrics : list, optional
             Metrics used in the model performance section. The metrics parameter should be a list
             of dict. Each dict contains they following keys :
             'path' (path to the metric function, ex: 'sklearn.metrics.mean_absolute_error'),
@@ -1068,6 +1152,10 @@ class SmartExplainer:
         notebook_path : str, optional
             Path to the notebook used to generate the report. If None, the Shapash base report
             notebook will be used.
+        kernel_name : str, optional
+            Name of the kernel used to generate the report. This parameter can be usefull if
+            you have multiple jupyter kernels and that the method does not use the right kernel
+            by default.
 
         Examples
         --------
@@ -1091,6 +1179,10 @@ class SmartExplainer:
                 ]
             )
         """
+        check_report_requirements()
+        # Avoid Import Errors with requirements specific to the Shapash Report
+        from shapash.report.generation import execute_report, export_and_save_report
+
         rm_working_dir = False
         if not working_dir:
             working_dir = tempfile.mkdtemp()
@@ -1112,7 +1204,8 @@ class SmartExplainer:
                 title_description=title_description,
                 metrics=metrics
             ),
-            notebook_path=notebook_path
+            notebook_path=notebook_path,
+            kernel_name=kernel_name
         )
         export_and_save_report(working_dir=working_dir, output_file=output_file)
 
