@@ -2,17 +2,21 @@
 Smart plotter module
 """
 import warnings
+from numbers import Number
 import random
 import copy
 import numpy as np
 import pandas as pd
 from plotly import graph_objs as go
 import plotly.express as px
+from plotly.subplots import make_subplots
 from plotly.offline import plot
 from shapash.manipulation.select_lines import select_lines
-from shapash.manipulation.summarize import compute_features_import
+from shapash.manipulation.summarize import compute_features_import, project_feature_values_1d, compute_corr
 from shapash.utils.utils import add_line_break, truncate_str, compute_digit_number, add_text, \
-    maximum_difference_sort_value, compute_sorted_variables_interactions_list_indices
+    maximum_difference_sort_value, compute_sorted_variables_interactions_list_indices, \
+    compute_top_correlations_features
+from shapash.webapp.utils.utils import round_to_k
 
 
 class SmartPlotter:
@@ -132,6 +136,11 @@ class SmartPlotter:
             'rgba(116, 1, 179, 0.9)',
         ]
 
+        self.groups_colors = [
+            px.colors.qualitative.T10[1],
+            px.colors.qualitative.G10[9]
+        ]
+
         self.round_digit = None
 
         self.interactions_col_scale = ["rgb(175, 169, 157)", "rgb(255, 255, 255)", "rgb(255, 77, 7)"]
@@ -272,6 +281,7 @@ class SmartPlotter:
                      proba_values=None,
                      col_modality=None,
                      col_scale=None,
+                     metadata=None,
                      addnote=None,
                      subtitle=None,
                      width=900,
@@ -320,17 +330,34 @@ class SmartPlotter:
         if pred is not None:
             hv_text = [f"Id: {x}<br />Predict: {y}" for x, y in zip(feature_values.index, pred.values.flatten())]
         else:
-            hv_text = [f"Id: {x}<br />" for x in feature_values.index]
+            hv_text = [f"Id: {x}" for x in feature_values.index]
+
+        if metadata:
+            metadata = {k: [round_to_k(x, 3) if isinstance(x, Number) else x for x in v]
+                        for k, v in metadata.items()}
+            text_groups_features = np.swap = np.array([col_values for col_values in metadata.values()])
+            text_groups_features = np.swapaxes(text_groups_features, 0, 1)
+            text_groups_features_keys = list(metadata.keys())
+            hovertemplate = '<b>%{hovertext}</b><br />' + \
+                            'Contribution: %{y:.4f} <br />' + \
+                            '<br />'.join([
+                                '{}: %{{text[{}]}}'.format(text_groups_features_keys[i], i)
+                                for i in range(len(text_groups_features_keys))
+                            ]) + '<extra></extra>'
+        else:
+            hovertemplate = '<b>%{hovertext}</b><br />' +\
+                            f'{feature_name} : ' +\
+                            '%{x}<br />Contribution: %{y:.4f}<extra></extra>'
+            text_groups_features = None
 
         fig.add_scatter(
             x=feature_values.values.flatten(),
             y=contributions.values.flatten(),
             mode='markers',
             hovertext=hv_text,
-            hovertemplate='<b>%{hovertext}</b><br />' +
-                          f'{feature_name} : ' +
-                          '%{x}<br />Contribution: %{y:.4f}<extra></extra>',
-            customdata=contributions.index.values
+            hovertemplate=hovertemplate,
+            customdata=feature_values.index.values,
+            text=text_groups_features
         )
 
         self._update_contributions_fig(fig=fig,
@@ -508,6 +535,7 @@ class SmartPlotter:
     def plot_features_import(self,
                              feature_imp1,
                              feature_imp2=None,
+                             title='Features Importance',
                              addnote=None,
                              subtitle=None,
                              width=900,
@@ -523,6 +551,8 @@ class SmartPlotter:
             Feature importance computed with every rows
         feature_imp2 : pd.Series, optional (default: None)
             The contributions associate
+        title : str
+            Title of the plot, default set to 'Features Importance'
         addnote : String (default: None)
             Specify a note to display
         subtitle : String (default: None)
@@ -537,7 +567,6 @@ class SmartPlotter:
             open automatically the plot
         """
         dict_t = copy.deepcopy(self.dict_title)
-        title = "Features Importance"
         topmargin = 80
         if subtitle or addnote:
             title += f"<span style='font-size: 12px;'><br />{add_text([subtitle, addnote], sep=' - ')}</span>"
@@ -551,6 +580,18 @@ class SmartPlotter:
         dict_style_bar2 = self.dict_featimp_colors[2]
         dict_yaxis['text'] = None
 
+        # Change bar color for groups of features
+        marker_color = [
+            self.groups_colors[0]
+            if (
+                    self.explainer.features_groups is not None
+                    and self.explainer.inv_features_dict.get(f.replace("<b>", "").replace("</b>", ""))
+                    in self.explainer.features_groups.keys()
+            )
+            else dict_style_bar1["color"]
+            for f in feature_imp1.index
+        ]
+
         layout = go.Layout(
             barmode='group',
             template='none',
@@ -573,7 +614,8 @@ class SmartPlotter:
             y=feature_imp1.index,
             orientation='h',
             name='Global',
-            marker=dict_style_bar1
+            marker=dict_style_bar1,
+            marker_color=marker_color
         )
         if feature_imp2 is not None:
             bar2 = go.Bar(
@@ -672,17 +714,41 @@ class SmartPlotter:
         )
         bars = []
         for num, expl in enumerate(list(zip(var_dict, x_val, contrib))):
+            group_name = None
             if expl[1] == '':
                 ylabel = '<i>{}</i>'.format(expl[0])
                 hoverlabel = '<b>{}</b>'.format(expl[0])
             else:
-                hoverlabel = '<b>{} :</b><br />{}'.format(add_line_break(expl[0], 40, maxlen=120),
-                                                          add_line_break(expl[1], 40, maxlen=160))
-                if len(contrib) <= yaxis_max_label:
-                    ylabel = '<b>{} :</b><br />{}'.format(truncate_str(expl[0], 45), truncate_str(expl[1], 45))
+                # If bar is a group of features, hovertext includes the values of the features of the group
+                # And color changes
+                if (self.explainer.features_groups is not None
+                        and self.explainer.inv_features_dict.get(expl[0]) in self.explainer.features_groups.keys()
+                        and len(index_value) > 0):
+                    group_name = self.explainer.inv_features_dict.get(expl[0])
+                    feat_groups_values = self.explainer.x_pred[self.explainer.features_groups[group_name]]\
+                                                       .loc[index_value[0]]
+                    hoverlabel = '<br />'.join([
+                        '<b>{} :</b>{}'.format(add_line_break(self.explainer.features_dict.get(f_name, f_name),
+                                                              40, maxlen=120),
+                                               add_line_break(f_value, 40, maxlen=160))
+                        for f_name, f_value in feat_groups_values.to_dict().items()
+                    ])
+                else:
+                    hoverlabel = '<b>{} :</b><br />{}'.format(add_line_break(expl[0], 40, maxlen=120),
+                                                              add_line_break(expl[1], 40, maxlen=160))
+                if len(contrib) <= yaxis_max_label and (
+                        self.explainer.features_groups is None
+                        # We don't want to display label values for t-sne projected values of groups of features.
+                        or (
+                                self.explainer.features_groups is not None
+                                and self.explainer.inv_features_dict.get(expl[0])
+                                not in self.explainer.features_groups.keys()
+                        )
+                ):
+                        ylabel = '<b>{} :</b><br />{}'.format(truncate_str(expl[0], 45), truncate_str(expl[1], 45))
+
                 else:
                     ylabel = ('<b>{}</b>'.format(truncate_str(expl[0], maxlen=45)))
-
             contrib_value = expl[2]
             # colors
             if contrib_value >= 0:
@@ -690,12 +756,19 @@ class SmartPlotter:
             else:
                 color = -1 if expl[1] != '' else -2
 
+            # If the bar is a group of features we modify the color
+            if group_name is not None:
+                bar_color = self.groups_colors[0] if color == 1 else self.groups_colors[1]
+            else:
+                bar_color = dict_local_plot_colors[color]['color']
+
             barobj = go.Bar(
                 x=[contrib_value],
                 y=[ylabel],
                 customdata=[hoverlabel],
                 orientation='h',
                 marker=dict_local_plot_colors[color],
+                marker_color=bar_color,
                 showlegend=False,
                 hovertemplate='%{customdata}<br />Contribution: %{x:.4f}<extra></extra>'
             )
@@ -839,7 +912,13 @@ class SmartPlotter:
             else:
                 value = None
         elif self.explainer._case == "regression":
-            value = self.explainer.model.predict(self.explainer.x_init.loc[index].to_frame().T)[0]
+            if self.explainer.y_pred is not None:
+                value = self.explainer.y_pred.loc[index]
+            else:
+                value = self.explainer.model.predict(self.explainer.x_init.loc[[index]])[0]
+
+        if isinstance(value, pd.Series):
+            value = value.values[0]
 
         return value
 
@@ -850,6 +929,7 @@ class SmartPlotter:
                    label=None,
                    show_masked=True,
                    show_predict=True,
+                   display_groups=None,
                    yaxis_max_label=12,
                    width=900,
                    height=550,
@@ -886,6 +966,10 @@ class SmartPlotter:
             show predict or predict proba value
         yaxis_max_label: int
             Maximum number of variables to display labels on the y axis
+        display_groups : bool (default: None)
+            Whether or not to display groups of features. This option is
+            only useful if groups of features are declared when compiling
+            SmartExplainer object.
         width : Int (default: 900)
             Plotly figure - layout width
         height : Int (default: 550)
@@ -904,6 +988,11 @@ class SmartPlotter:
         --------
         >>> xpl.plot.local_plot(row_num=0)
         """
+        display_groups = True if (display_groups is not False and self.explainer.features_groups is not None) else False
+        if display_groups:
+            data = self.explainer.data_groups
+        else:
+            data = self.explainer.data
         # checking args
         if sum(arg is not None for arg in [query, row_num, index]) != 1:
             raise ValueError(
@@ -931,8 +1020,15 @@ class SmartPlotter:
 
         else:
             # apply filter if the method have not yet been asked in order to limit the number of feature to display
-            if not hasattr(self.explainer, 'mask_params'):
-                self.explainer.filter(max_contrib=20)
+            if (
+                not hasattr(self.explainer, "mask_params")  # If the filter method has not been called yet
+                # Or if the already computed mask was not updated with current display_groups parameter
+                or (isinstance(data["contrib_sorted"], pd.DataFrame)
+                    and len(data["contrib_sorted"].columns) != len(self.explainer.mask.columns))
+                or (isinstance(data["contrib_sorted"], list)
+                    and len(data["contrib_sorted"][0].columns) != len(self.explainer.mask[0].columns))
+            ):
+                self.explainer.filter(max_contrib=20, display_groups=display_groups)
 
             if self.explainer._case == "classification":
                 if label is None:
@@ -940,9 +1036,9 @@ class SmartPlotter:
 
                 label_num, _, label_value = self.explainer.check_label_name(label)
 
-                contrib = self.explainer.data['contrib_sorted'][label_num]
-                x_val = self.explainer.data['x_sorted'][label_num]
-                var_dict = self.explainer.data['var_dict'][label_num]
+                contrib = data['contrib_sorted'][label_num]
+                x_val = data['x_sorted'][label_num]
+                var_dict = data['var_dict'][label_num]
 
                 if show_predict is True:
                     pred = self.local_pred(line[0], label_num)
@@ -952,9 +1048,9 @@ class SmartPlotter:
                         subtitle = f"Response: <b>{label_value}</b> - Proba: <b>{pred:.4f}</b>"
 
             elif self.explainer._case == "regression":
-                contrib = self.explainer.data['contrib_sorted']
-                x_val = self.explainer.data['x_sorted']
-                var_dict = self.explainer.data['var_dict']
+                contrib = data['contrib_sorted']
+                x_val = data['x_sorted']
+                var_dict = data['var_dict']
                 label_num = None
                 if show_predict is True:
                     pred_value = self.local_pred(line[0])
@@ -969,10 +1065,12 @@ class SmartPlotter:
             var_dict, x_val, contrib = self.get_selection(line, var_dict, x_val, contrib)
             var_dict, x_val, contrib = self.apply_mask_one_line(line, var_dict, x_val, contrib, label=label_num)
             # use label of each column
-            var_dict = [self.explainer.features_dict[self.explainer.columns_dict[x]] for x in var_dict]
+            if display_groups:
+                var_dict = [self.explainer.features_dict[self.explainer.x_pred_groups.columns[x]] for x in var_dict]
+            else:
+                var_dict = [self.explainer.features_dict[self.explainer.columns_dict[x]] for x in var_dict]
             if show_masked:
                 var_dict, x_val, contrib = self.check_masked_contributions(line, var_dict, x_val, contrib, label=label_num)
-
             # Filtering all negative or positive contrib if specify in mask
             exclusion = []
             if hasattr(self.explainer, 'mask_params'):
@@ -1059,16 +1157,26 @@ class SmartPlotter:
 
         if not isinstance(col, (str, int)):
             raise ValueError('parameter col must be string or int.')
+        if hasattr(self.explainer, 'inv_features_dict'):
+            col = self.explainer.inv_features_dict.get(col, col)
+        col_is_group = self.explainer.features_groups and col in self.explainer.features_groups.keys()
 
-        col_id = self.explainer.check_features_name([col])[0]
-        col_name = self.explainer.columns_dict[col_id]
-
-        col_value_count = self.explainer.features_desc[col_name]
-
-        if self.explainer.features_dict:
-            col_label = self.explainer.features_dict[col_name]
+        # Case where col is a group of features
+        if col_is_group:
+            contributions = self.explainer.contributions_groups
+            col_label = self.explainer.features_dict[col]
+            col_name = self.explainer.features_groups[col]  # Here col_name is actually a list of features
+            col_value_count = self.explainer.features_desc[col]
         else:
-            col_label = col_name
+            contributions = self.explainer.contributions
+            col_id = self.explainer.check_features_name([col])[0]
+            col_name = self.explainer.columns_dict[col_id]
+            col_value_count = self.explainer.features_desc[col_name]
+
+            if self.explainer.features_dict:
+                col_label = self.explainer.features_dict[col_name]
+            else:
+                col_label = col_name
 
         # Sampling
         if selection is None:
@@ -1086,8 +1194,7 @@ class SmartPlotter:
                 list_ind = random.sample(selection, max_points)
                 addnote = "Length of random Subset : "
         else:
-            ValueError('parameter selection must be a list')
-
+            raise ValueError('parameter selection must be a list')
         if addnote is not None:
             addnote = add_text([addnote,
                                 f"{len(list_ind)} ({int(np.round(100 * len(list_ind) / self.explainer.x_pred.shape[0]))}%)"],
@@ -1100,7 +1207,7 @@ class SmartPlotter:
 
         # Classification Case
         if self.explainer._case == "classification":
-            subcontrib = self.explainer.contributions[label_num]
+            subcontrib = contributions[label_num]
             if self.explainer.y_pred is not None:
                 col_value = self.explainer._classes[label_num]
             subtitle = f"Response: <b>{label_value}</b>"
@@ -1119,7 +1226,7 @@ class SmartPlotter:
 
         # Regression Case - color scale
         elif self.explainer._case == "regression":
-            subcontrib = self.explainer.contributions
+            subcontrib = contributions
             if self.explainer.y_pred is not None:
                 if not hasattr(self, "pred_colorscale"):
                     self.pred_colorscale = self.tuning_colorscale(self.explainer.y_pred)
@@ -1127,10 +1234,34 @@ class SmartPlotter:
 
         # Subset
         if self.explainer.postprocessing_modifications:
-            feature_values = self.explainer.x_contrib_plot.loc[list_ind, col_name].to_frame()
+            feature_values = self.explainer.x_contrib_plot.loc[list_ind, col_name]
         else:
-            feature_values = self.explainer.x_pred.loc[list_ind, col_name].to_frame()
-        contrib = subcontrib.loc[list_ind, col_name].to_frame()
+            feature_values = self.explainer.x_pred.loc[list_ind, col_name]
+
+        if col_is_group:
+            feature_values = project_feature_values_1d(feature_values, col, self.explainer.x_pred,
+                                                       self.explainer.x_init, self.explainer.preprocessing,
+                                                       features_dict=self.explainer.features_dict)
+            contrib = subcontrib.loc[list_ind, col].to_frame()
+            if self.explainer.features_imp is None:
+                self.explainer.compute_features_import()
+            features_imp = self.explainer.features_imp if isinstance(self.explainer.features_imp, pd.Series) \
+                else self.explainer.features_imp[0]
+            top_features_of_group = features_imp.loc[self.explainer.features_groups[col]] \
+                                                .sort_values(ascending=False)[:4].index  # Displaying top 4 features
+            metadata = {
+                self.explainer.features_dict[f_name]: self.explainer.x_pred[f_name]
+                for f_name in top_features_of_group
+            }
+            text_group = "Features values were projected on the x axis using t-SNE"
+            if addnote is not None:
+                addnote = add_text([addnote, text_group], sep=' - ')
+            else:
+                addnote = text_group
+        else:
+            contrib = subcontrib.loc[list_ind, col_name].to_frame()
+            metadata = None
+        feature_values = feature_values.to_frame()
 
         if self.explainer.y_pred is not None:
             y_pred = self.explainer.y_pred.loc[list_ind]
@@ -1149,7 +1280,7 @@ class SmartPlotter:
         # selecting the best plot : Scatter, Violin?
         if col_value_count > violin_maxf:
             fig = self.plot_scatter(feature_values, contrib, col_label, y_pred, proba_values, col_value, col_scale,
-                                    addnote,
+                                    metadata, addnote,
                                     subtitle, width, height, file_name, auto_open)
         else:
             fig = self.plot_violin(feature_values, contrib, col_label, y_pred, proba_values, col_value, col_scale,
@@ -1162,6 +1293,8 @@ class SmartPlotter:
                             max_features=20,
                             selection=None,
                             label=-1,
+                            group_name=None,
+                            display_groups=True,
                             force=False,
                             width=900,
                             height=500,
@@ -1191,6 +1324,15 @@ class SmartPlotter:
         label: integer or string (default -1)
             If the label is of string type, check if it can be changed to integer to select the
             good dataframe object.
+        group_name : str (optional, default None)
+            Allows to display the features importance of the variables that are grouped together
+            inside a group of features.
+            This parameter is only available if the SmartExplainer object has been compiled using
+            the features_groups optional parameter and should correspond to a key of
+            features_groups dictionary.
+        display_groups : bool (default True)
+            If groups of features are declared in SmartExplainer object, this parameter allows to
+            specify whether or not to display them.
         force: bool (optional, default False)
             force == True, force the compute features importance if it's already done
         width : Int (default: 900)
@@ -1213,13 +1355,38 @@ class SmartPlotter:
 
         self.explainer.compute_features_import(force=force)
         subtitle = None
+        title = 'Features Importance'
+        display_groups = self.explainer.features_groups is not None and display_groups
+
+        if display_groups:
+            if group_name:  # Case where we have groups of features and we want to display only features inside a group
+                if group_name not in self.explainer.features_groups.keys():
+                    raise ValueError(f"group_name parameter : {group_name} is not in features_groups keys. "
+                                     f"Possible values are : {list(self.explainer.features_groups.keys())}")
+                title += f' - {truncate_str(self.explainer.features_dict.get(group_name), 20)}'
+                if isinstance(self.explainer.features_imp, list):
+                    features_importance = [
+                        label_feat_imp.loc[label_feat_imp.index.isin(self.explainer.features_groups[group_name])]
+                        for label_feat_imp in self.explainer.features_imp
+                    ]
+                else:
+                    features_importance = self.explainer.features_imp.loc[
+                        self.explainer.features_imp.index.isin(self.explainer.features_groups[group_name])
+                    ]
+                contributions = self.explainer.contributions
+            else:
+                features_importance = self.explainer.features_imp_groups
+                contributions = self.explainer.contributions_groups
+        else:
+            features_importance = self.explainer.features_imp
+            contributions = self.explainer.contributions
 
         # classification
         if self.explainer._case == "classification":
             label_num, _, label_value = self.explainer.check_label_name(label)
-            global_feat_imp = self.explainer.features_imp[label_num].tail(max_features)
+            global_feat_imp = features_importance[label_num].tail(max_features)
             if selection is not None:
-                subset = self.explainer.contributions[label_num].loc[selection]
+                subset = contributions[label_num].loc[selection]
                 subset_feat_imp = compute_features_import(subset)
                 subset_feat_imp = subset_feat_imp.reindex(global_feat_imp.index)
             else:
@@ -1227,9 +1394,9 @@ class SmartPlotter:
             subtitle = f"Response: <b>{label_value}</b>"
         # regression
         elif self.explainer._case == "regression":
-            global_feat_imp = self.explainer.features_imp.tail(max_features)
+            global_feat_imp = features_importance.tail(max_features)
             if selection is not None:
-                subset = self.explainer.contributions.loc[selection]
+                subset = contributions.loc[selection]
                 subset_feat_imp = compute_features_import(subset)
                 subset_feat_imp = subset_feat_imp.reindex(global_feat_imp.index)
             else:
@@ -1250,7 +1417,22 @@ class SmartPlotter:
                                sep=" - ")
 
         global_feat_imp.index = global_feat_imp.index.map(self.explainer.features_dict)
-        fig = self.plot_features_import(global_feat_imp, subset_feat_imp, addnote,
+        if display_groups:
+            # Bold font for groups of features
+            global_feat_imp.index = [
+                '<b>' + str(f) + '</b>'
+                if self.explainer.inv_features_dict.get(f) in self.explainer.features_groups.keys()
+                else str(f) for f in global_feat_imp.index
+            ]
+
+            if subset_feat_imp is not None:
+                subset_feat_imp.index = [
+                    '<b>' + str(f) + '</b>'
+                    if self.explainer.inv_features_dict.get(f) in self.explainer.features_groups.keys()
+                    else str(f) for f in subset_feat_imp.index
+                ]
+
+        fig = self.plot_features_import(global_feat_imp, subset_feat_imp, title, addnote,
                                         subtitle, width, height, file_name, auto_open)
         return fig
 
@@ -2003,3 +2185,141 @@ class SmartPlotter:
             plot(fig, filename=file_name, auto_open=auto_open)
 
         return fig
+
+    def correlations(
+            self,
+            df=None,
+            max_features=20,
+            features_to_hide=None,
+            facet_col=None,
+            how='phik',
+            width=900,
+            height=500,
+            file_name=None,
+            auto_open=False
+    ):
+        """
+        Correlations matrix heatmap plot.
+        The method can use phik or pearson correlations.
+        The correlations computed can be changed using the parameter 'how'.
+
+        Parameters
+        ----------
+        df : pd.DataFrame, optional
+            DataFrame for which we want to compute correlations. Will use x_pred by default.
+        max_features : int (default: 10)
+            Max number of features to show on the matrix.
+        features_to_hide : list (optional)
+            List of features that will not appear on the graph
+        facet_col : str (optional)
+            Name of the column used to split the graph in two (or more) plots. One correlation
+            subplot will be computed for each value of this column.
+        how : str (default: 'phik')
+            Correlation method used. 'phik' or 'pearson' are possible values. 'phik' is used by default.
+        width : Int (default: 900)
+            Plotly figure - layout width
+        height : Int (default: 600)
+            Plotly figure - layout height
+        file_name: string (optional)
+            File name to use to save the plotly bar chart. If None the bar chart will not be saved.
+        auto_open: Boolean (optional)
+            Indicate whether to open the bar plot or not.
+
+        Returns
+        -------
+        go.Figure
+
+        Example
+        --------
+        >>> xpl.plot.correlations()
+        """
+
+        if features_to_hide is None:
+            features_to_hide = []
+
+        if df is None:
+            # Use x_pred by default
+            df = self.explainer.x_pred
+
+        if facet_col:
+            features_to_hide += [facet_col]
+
+        # We use phik by default as it is a convenient method for numeric and categorical data
+        if how == 'phik':
+            try:
+                from phik import phik_matrix
+                compute_method = 'phik'
+            except (ImportError, ModuleNotFoundError):
+                warnings.warn('Cannot compute phik correlations. Install phik using "pip install phik".', UserWarning)
+                compute_method = "pearson"
+        else:
+            compute_method = how
+
+        hovertemplate = '<b>%{text}<br />Correlation: %{z}</b><extra></extra>'
+
+        list_features = []
+        if facet_col:
+            facet_col_values = sorted(df[facet_col].unique(), reverse=True)
+            fig = make_subplots(
+                rows=1,
+                cols=df[facet_col].nunique(),
+                subplot_titles=[t + " correlation" for t in facet_col_values],
+                horizontal_spacing=0.15
+            )
+            # Used for the Shapash report to get train then test set
+            for i, col_v in enumerate(facet_col_values):
+                corr = compute_corr(df.loc[df[facet_col] == col_v].drop(features_to_hide, axis=1), compute_method)
+
+                # Keep the same list of features for each subplot
+                if len(list_features) == 0:
+                    list_features = compute_top_correlations_features(corr=corr, max_features=max_features)
+
+                fig.add_trace(
+                    go.Heatmap(
+                        z=corr.loc[list_features, list_features].round(2).values,
+                        x=list_features,
+                        y=list_features,
+                        coloraxis='coloraxis',
+                        text=[[f'Feature 1: {self.explainer.features_dict.get(y, y)} <br />'
+                               f'Feature 2: {self.explainer.features_dict.get(x, x)}' for x in list_features]
+                              for y in list_features],
+                        hovertemplate=hovertemplate,
+                    ), row=1, col=i+1)
+
+        else:
+            corr = compute_corr(df.drop(features_to_hide, axis=1), compute_method)
+            list_features = compute_top_correlations_features(corr=corr, max_features=max_features)
+
+            fig = go.Figure(go.Heatmap(
+                        z=corr.loc[list_features, list_features].round(2).values,
+                        x=list_features,
+                        y=list_features,
+                        coloraxis='coloraxis',
+                        text=[[f'Feature 1: {self.explainer.features_dict.get(y, y)} <br />'
+                               f'Feature 2: {self.explainer.features_dict.get(x, x)}' for x in list_features]
+                              for y in list_features],
+                        hovertemplate=hovertemplate,
+                    ))
+
+        title = f'Correlation ({compute_method})'
+        if len(list_features) < len(df.drop(features_to_hide, axis=1).columns):
+            subtitle = f"Top {len(list_features)} correlations"
+            title += f"<span style='font-size: 12px;'><br />{subtitle}</span>"
+        dict_t = copy.deepcopy(self.dict_title)
+        dict_t['text'] = title
+
+        fig.update_layout(
+            coloraxis=dict(colorscale=['rgb(255, 255, 255)'] + self.init_colorscale[5:-1]),
+            showlegend=True,
+            title=dict_t,
+            width=width,
+            height=height
+        )
+
+        fig.update_yaxes(automargin=True)
+        fig.update_xaxes(automargin=True)
+
+        if file_name:
+            plot(fig, filename=file_name, auto_open=auto_open)
+
+        return fig
