@@ -16,9 +16,9 @@ from shapash.utils.utils import get_project_root, truncate_str
 from shapash.report.visualisation import print_md, print_html, print_css_style, convert_fig_to_html, \
     print_javascript_misc
 from shapash.report.data_analysis import perform_global_dataframe_analysis, perform_univariate_dataframe_analysis
-from shapash.report.plots import generate_fig_univariate, generate_confusion_matrix_plot, \
-    generate_correlation_matrix_fig
-from shapash.report.common import series_dtype, get_callable, compute_col_types
+from shapash.report.plots import generate_fig_univariate, generate_confusion_matrix_plot
+from shapash.report.common import series_dtype, get_callable, compute_col_types, VarType, display_value
+from shapash.webapp.utils.utils import round_to_k
 
 logging.basicConfig(level=logging.INFO)
 
@@ -106,7 +106,6 @@ class ProjectReport:
                     if key == 'use_proba_values' and not isinstance(metric['use_proba_values'], bool):
                         raise ValueError('"use_proba_values" metric key expects a boolean value.')
 
-
     @staticmethod
     def _get_values_and_name(
             y: Optional[Union[pd.DataFrame, pd.Series, list]],
@@ -277,9 +276,15 @@ class ProjectReport:
                         group_id='target'
                     )
         if multivariate_analysis:
-            print_md("### Mutlivariate analysis")
-            fig_corr = generate_correlation_matrix_fig(self.df_train_test, max_features=20)
-            print_html(convert_fig_to_html(fig=fig_corr))
+            print_md("### Multivariate analysis")
+            fig_corr = self.explainer.plot.correlations(
+                self.df_train_test,
+                facet_col='data_train_test',
+                max_features=20,
+                width=900,
+                height=500,
+            )
+            print_html(plotly.io.to_html(fig_corr))
         print_md('---')
 
     def _display_dataset_analysis_global(self):
@@ -306,7 +311,10 @@ class ProjectReport:
 
         univariate_template = template_env.get_template("univariate.html")
         univariate_features_desc = list()
-        for col in df.drop(col_splitter, axis=1).columns:
+        list_cols_labels = [self.explainer.features_dict.get(col, col)
+                            for col in df.drop(col_splitter, axis=1).columns.to_list()]
+        for col_label in sorted(list_cols_labels):
+            col = self.explainer.inv_features_dict.get(col_label, col_label)
             fig = generate_fig_univariate(df_all=df, col=col, hue=col_splitter, type=col_types[col])
             df_col_stats = self._stats_to_table(
                 test_stats=test_stats_univariate[col],
@@ -317,7 +325,7 @@ class ProjectReport:
                 'feature_index': int(self.explainer.inv_columns_dict.get(col, 0)),
                 'name': col,
                 'type': str(series_dtype(df[col])),
-                'description': self.explainer.features_dict.get(col, ''),
+                'description': col_label,
                 'table': df_col_stats.to_html(classes="greyGridTable"),
                 'image': convert_fig_to_html(fig)
             })
@@ -350,7 +358,10 @@ class ProjectReport:
             fig_features_importance = self.explainer.plot.features_importance(label=label)
 
             explain_contrib_data = list()
-            for feature in self.col_names:
+            list_cols_labels = [self.explainer.features_dict.get(col, col)
+                                for col in self.col_names]
+            for feature_label in sorted(list_cols_labels):
+                feature = self.explainer.inv_features_dict.get(feature_label, feature_label)
                 fig = self.explainer.plot.contribution_plot(feature, label=label, max_points=200)
                 explain_contrib_data.append({
                     'feature_index': int(self.explainer.inv_columns_dict[feature]),
@@ -434,7 +445,8 @@ class ProjectReport:
                     logging.info(f"Could not compute following metric : {metric['path']}. \n{e}")
                     continue
                 if isinstance(res, Number):
-                    print_md(f"**{metric['name']} :** {round(res, 2)}")
+                    res = display_value(round_to_k(res, 3))
+                    print_md(f"**{metric['name']} :** {res}")
                 elif isinstance(res, (list, tuple, np.ndarray)):
                     print_md(f"**{metric['name']} :**")
                     print_html(pd.DataFrame(res).to_html(classes="greyGridTable"))
