@@ -3,6 +3,7 @@ Main class of Web application Shapash
 """
 import dash
 import dash_table
+import dash_daq as daq
 from dash import no_update
 from dash.exceptions import PreventUpdate
 import dash_bootstrap_components as dbc
@@ -85,6 +86,7 @@ class SmartApp:
             self.max_threshold = int(self.explainer.contributions.applymap(lambda x: round_to_k(x, k=1)).max().max())
         self.list_index = []
         self.subset = None
+        self.last_click_data = None
 
         # DATA
         self.dataframe = pd.DataFrame()
@@ -146,7 +148,7 @@ class SmartApp:
                 std = self.dataframe[col].std()
                 if std != 0:
                     digit = max(round(log10(1 / std) + 1) + 2, 0)
-                    self.round_dataframe[col] = self.dataframe[col].map(f'{{:.{digit}f}}'.format)
+                    self.round_dataframe[col] = self.dataframe[col].map(f'{{:.{digit}f}}'.format).astype(float)
 
     def init_components(self):
         """
@@ -214,6 +216,30 @@ class SmartApp:
 
         self.components['menu'] = dbc.Row(
             [
+                dbc.Col(
+                    [
+                        html.Div(
+                            daq.BooleanSwitch(
+                                id='bool_groups',
+                                on=True,
+                                style={'display': 'none'} if self.explainer.features_groups is None else {},
+                                color='rgba(244, 192, 0, 1.0)',
+                                label={
+                                    'label': 'Groups',
+                                    'style': {
+                                        'fontSize': 18,
+                                        'color': 'rgb(244, 192, 0)',
+                                        'fontWeight': 'bold',
+                                        "margin-left": "5px"
+                                    },
+                                },
+                                labelPosition="right"
+                            ),
+                            style={"margin-right": "35px"}
+                        )
+                    ],
+                    width="auto", align="center",
+                ),
                 dbc.Col(
                     [
                         html.H4(
@@ -229,12 +255,12 @@ class SmartApp:
                     dbc.Collapse(
                         dbc.FormGroup(
                             [
-                                dbc.Label("Class to analyse", style={'color': 'white', 'margin': '0px 5px'}),
+                                dbc.Label("Class", style={'color': 'white', 'margin': '0px 5px'}),
                                 dcc.Dropdown(
                                     id="select_label",
                                     options=[], value=None,
                                     clearable=False, searchable=False,
-                                    style={"verticalAlign": "middle", "zIndex": '1010', "min-width": '200px'}
+                                    style={"verticalAlign": "middle", "zIndex": '1010', "min-width": '100px'}
                                 )
                             ],
                             row=True, style={"margin": "0px 0px 0px 5px", "align-items": "center"}
@@ -735,6 +761,7 @@ class SmartApp:
         @app.callback(
             [
                 Output('dataset', 'data'),
+                Output('dataset', 'tooltip_data'),
                 Output('dataset', 'columns'),
                 Output('dataset', 'active_cell'),
             ],
@@ -784,8 +811,19 @@ class SmartApp:
                     inplace=False
                 )
             self.components['table']['dataset'].data = df.to_dict('records')
+            self.components['table']['dataset'].tooltip_data = [
+                {
+                    column: {'value': str(value), 'type': 'text'}
+                    for column, value in row.items()
+                } for row in df.to_dict('rows')
+            ]
 
-            return self.components['table']['dataset'].data, columns, active_cell
+            return (
+                self.components['table']['dataset'].data,
+                self.components['table']['dataset'].tooltip_data,
+                columns,
+                active_cell,
+            )
 
         @app.callback(
             [
@@ -795,42 +833,67 @@ class SmartApp:
             [
                 Input('select_label', 'value'),
                 Input('dataset', 'data'),
-                Input('modal', 'is_open')
+                Input('modal', 'is_open'),
+                Input('card_global_feature_importance', 'n_clicks'),
+                Input('bool_groups', 'on')
             ],
-            [State('global_feature_importance', 'clickData'),
-             State('dataset', "filter_query"),
-             State('features', 'value')]
+            [
+                State('global_feature_importance', 'clickData'),
+                State('dataset', "filter_query"),
+                State('features', 'value')
+            ]
         )
-        def update_feature_importance(label, data, is_open, clickData, filter_query, features):
+        def update_feature_importance(label, data, is_open, n_clicks, bool_group, clickData, filter_query, features):
             """
             update feature importance plot according to selected label and dataset state.
             """
             ctx = dash.callback_context
+            selected_feature = self.explainer.inv_features_dict.get(
+                clickData['points'][0]['label'].replace('<b>', '').replace('</b>', '')
+            ) if clickData else None
             if ctx.triggered[0]['prop_id'] == 'modal.is_open':
-
                 if is_open:
                     raise PreventUpdate
                 else:
-
                     self.settings['features'] = features
                     self.settings_ini['features'] = self.settings['features']
-
             elif ctx.triggered[0]['prop_id'] == 'select_label.value':
                 self.label = label
             elif ctx.triggered[0]['prop_id'] == 'dataset.data':
                 self.list_index = [d['_index_'] for d in data]
+            elif (ctx.triggered[0]['prop_id'] == 'card_global_feature_importance.n_clicks'
+                  and self.explainer.features_groups):
+                # When we click twice on the same bar this will reset the graph
+                if self.last_click_data == clickData:
+                    selected_feature = None
+                list_sub_features = [f for group_features in self.explainer.features_groups.values()
+                                     for f in group_features]
+                if selected_feature in list_sub_features:
+                    self.last_click_data = clickData
+                    raise PreventUpdate
+                else:
+                    pass
+            elif ctx.triggered[0]['prop_id'] == 'bool_groups.on':
+                clickData = None  # We reset the graph and clicks if we toggle the button
             else:
+                self.last_click_data = clickData
                 raise PreventUpdate
 
+            group_name = selected_feature if (self.explainer.features_groups is not None
+                                              and selected_feature in self.explainer.features_groups.keys()) else None
             selection = self.list_index if filter_query else None
             self.components['graph']['global_feature_importance'].figure = \
-                self.explainer.plot.features_importance(max_features=features,
-                                                        selection=selection,
-                                                        label=self.label
-                                                        )
+                self.explainer.plot.features_importance(
+                    max_features=features,
+                    selection=selection,
+                    label=self.label,
+                    group_name=group_name,
+                    display_groups=bool_group
+                )
             self.components['graph']['global_feature_importance'].adjust_graph()
             self.components['graph']['global_feature_importance'].figure.layout.clickmode = 'event+select'
-            self.select_point('global_feature_importance', clickData)
+            if selected_feature and selected_feature not in self.explainer.features_groups.keys():
+                self.select_point('global_feature_importance', clickData)
 
             # font size can be adapted to screen size
             nb_car = max([len(self.components['graph']['global_feature_importance'].figure.data[0].y[i]) for i in
@@ -838,7 +901,7 @@ class SmartApp:
             self.components['graph']['global_feature_importance'].figure.update_layout(
                 yaxis=dict(tickfont={'size': min(round(500 / nb_car), 12)})
             )
-
+            self.last_click_data = clickData
             return self.components['graph']['global_feature_importance'].figure, clickData
 
         @app.callback(
@@ -871,7 +934,8 @@ class SmartApp:
                 self.label = label
             elif ctx.triggered[0]['prop_id'] == 'global_feature_importance.clickData':
                 if feature is not None:
-                    self.selected_feature = feature['points'][0]['label']
+                    # Removing bold
+                    self.selected_feature = feature['points'][0]['label'].replace('<b>', '').replace('</b>', '')
                     if feature['points'][0]['curveNumber'] == 0 and \
                             len(self.components['graph']['global_feature_importance'].figure['data']) == 2:
                         self.subset = self.list_index
@@ -996,6 +1060,7 @@ class SmartApp:
                 Input('select_label', 'value'),
                 Input('dataset', 'active_cell'),
                 Input('feature_selector', 'clickData'),
+                Input('bool_groups', 'on')
             ],
             [
                 State('index_id', 'value'),
@@ -1003,7 +1068,7 @@ class SmartApp:
             ]
         )
         def update_detail_feature(threshold, max_contrib, positive, negative, masked, label, cell,
-                                  click_data, index, data):
+                                  click_data, bool_group, index, data):
             """
             update local explanation plot according to app changes.
             """
@@ -1034,17 +1099,24 @@ class SmartApp:
             self.explainer.filter(threshold=threshold,
                                   features_to_hide=masked,
                                   positive=sign,
-                                  max_contrib=max_contrib)
+                                  max_contrib=max_contrib,
+                                  display_groups=bool_group)
             if np.issubdtype(type(self.explainer.x_pred.index[0]), np.dtype(int).type):
                 selected = int(selected)
-            self.components['graph']['detail_feature'].figure = self.explainer.plot.local_plot(index=selected,
-                                                                                               label=label,
-                                                                                               show_masked=True,
-                                                                                               yaxis_max_label=0)
+            self.components['graph']['detail_feature'].figure = self.explainer.plot.local_plot(
+                index=selected,
+                label=label,
+                show_masked=True,
+                yaxis_max_label=8,
+                display_groups=bool_group
+            )
             self.components['graph']['detail_feature'].adjust_graph(title_size_adjust=True)
             # font size can be adapted to screen size
-            nb_car = max([len(self.components['graph']['detail_feature'].figure.data[i].y[0]) for i in
-                          range(len(self.components['graph']['detail_feature'].figure.data))])
+            list_yaxis = [self.components['graph']['detail_feature'].figure.data[i].y[0] for i in
+                          range(len(self.components['graph']['detail_feature'].figure.data))]
+            # exclude new line with labels of y axis
+            list_yaxis = [x.split('<br />')[0] for x in list_yaxis]
+            nb_car = max([len(x) for x in list_yaxis])
             self.components['graph']['detail_feature'].figure.update_layout(
                 yaxis=dict(tickfont={'size': min(round(500 / nb_car), 12)})
             )
